package ch.std.jumpstart.rest;

import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import ch.std.jumpstart.jpa.City;
import ch.std.jumpstart.json.JSONServiceRequest;
import ch.std.jumpstart.json.JSONServiceResponse;
import ch.std.jumpstart.parser.impl.CityParser;
import ch.std.jumpstart.renderer.ListRenderer;
import ch.std.jumpstart.renderer.Renderer;
import ch.std.jumpstart.renderer.impl.CityRenderer;
import ch.std.jumpstart.repository.CityRepository;

// see https://spring.io/guides/tutorials/rest/
// see https://medium.com/@salisuwy/building-a-spring-boot-rest-api-part-ii-7ff1e4384b0b

@RestController
@RequestMapping("/rest/city")
public class CityController extends GenericRestController<Object> {

	Logger logger = LoggerFactory.getLogger(CityController.class);

	@Autowired
	CityRepository cityRepository;

	@Override
	public JSONServiceResponse executeGet(RestRequest restRequest) throws Exception {
		String value = restRequest.getStringRequestParameter("value", null);
		logger.info("get cities, value = " + value);
		List<City> cities = null;
		if (value == null) {
			cities = (List<City>) cityRepository.findAll();
		} else {
			cities = (List<City>) cityRepository.findByNameLike(value);
		}
		JSONServiceResponse jsonServiceResponse = new JSONServiceResponse();
		ListRenderer<?, City> renderer = new ListRenderer<Map<String, Object>, City>(new CityRenderer());
		jsonServiceResponse.setBodyValue("cities", renderer.render(cities));
		return jsonServiceResponse;
	}

	@Override
	public JSONServiceResponse executeGetById(Object id, RestRequest restRequest) throws Exception {
		Long lid = Long.parseLong(id.toString());
		City city = cityRepository.findById(lid).orElseThrow(() -> new CityNotFoundException(lid));
		JSONServiceResponse jsonServiceResponse = new JSONServiceResponse();
		Renderer<?, City> renderer = new CityRenderer();
		jsonServiceResponse.setBodyValue("city", renderer.render(city));
		return jsonServiceResponse;
	}

	@Override
	public JSONServiceResponse executePost(JSONServiceRequest jsonServiceRequest, RestRequest restRequest) throws Exception {
		JSONServiceResponse jsonServiceResponse = new JSONServiceResponse();
		City city = new CityParser().parse(jsonServiceRequest.getBodyMap("city"));
		// check if city already exists
		City existingCity = cityRepository.findByName(city.getName()).orElse(null);
		if (existingCity != null) {
			logger.info("city " + city + " is already exsting");
			jsonServiceResponse.setSuccess("already existing");
			jsonServiceResponse.setBodyValue("city", new CityRenderer().render(existingCity));
			return jsonServiceResponse;
		}
		cityRepository.save(city);
		logger.info("city " + city + " new created");
		jsonServiceResponse.setSuccess("new created");
		jsonServiceResponse.setBodyValue("city", new CityRenderer().render(city));
		return jsonServiceResponse;
	}
	
	@Override
	public JSONServiceResponse executePut(JSONServiceRequest jsonServiceRequest, RestRequest restRequest) throws Exception {
		JSONServiceResponse jsonServiceResponse = new JSONServiceResponse();
		return jsonServiceResponse;
	}
	
	@Override
	public JSONServiceResponse executePutById(Object id, RestRequest restRequest) throws Exception {
		JSONServiceResponse jsonServiceResponse = new JSONServiceResponse();
		return jsonServiceResponse;
	}
	
	public static class CityNotFoundException extends RuntimeException {

		private static final long serialVersionUID = -3515876169075383850L;

		public CityNotFoundException(Long id) {
			super("Could not find city " + id);
		}
	}
}
